local u = BittensGlobalTables.GetTable("BittensUtilities")
if u.SkipOrUpgrade(u, "Options Panels", 1) then
	return
end

local _G = _G
local CreateFrame = CreateFrame
local InterfaceOptions_AddCategory = InterfaceOptions_AddCategory
local ipairs = ipairs
local wipe = wipe

-- Add the functions apply() and refresh() to the returned frame.  They, along
-- with widgets in the frame, should use/modify the values in frame.Settings.
-- If you wish changes in widgets to apply immediately, call frame.refresh() 
-- after changing frame.Settings.
--
-- An empty table represents default values.
function u.CreateOptionsPanel(name, template, parent)	
	local frame = CreateFrame("Frame", nil, nil, template)
	frame.name = name
	
	function frame.Initialize(savedSettingsTableName)
		frame.SavedSettings = u.GetOrMakeTable(_G, savedSettingsTableName)
		frame.Settings = u.ShallowCopy(frame.SavedSettings)
		frame.apply()
	end
	
	function frame.AddLabel(text, template, x, y)
		local label = frame:CreateFontString(nil, nil, template)
		label:SetPoint("TOPLEFT", x, y)
		label:SetText(text)
		return y - label:GetHeight()
	end
	
	-- definition = {
	--     Name = string,
	--     Text = string,
	--     Default = boolean,
	-- }
	function frame.AddCheckBox(definition, x, y)
		local checkName = name .. definition.Name .. "CheckBox"
		local check = CreateFrame(
			"CheckButton", 
			checkName, 
			frame, 
			"InterfaceOptionsCheckButtonTemplate")
		check:SetPoint("TOPLEFT", x, y)
		_G[checkName .. "Text"]:SetText(definition.Text)
		check:SetScript("OnClick", function()
			frame.Settings[definition.Name] = not not check:GetChecked()
			frame.apply()
		end)
		u.GetOrMakeTable(frame, "CheckBoxes")[definition] = check
		return y - check:GetHeight()
	end
	
	function frame.AddCheckBoxGroup(label, definitions, x, y)
		y = frame.AddLabel(label, "GameFontNormalLarge", x, y)
		for _, definition in ipairs(definitions) do
			y = frame.AddCheckBox(definition, x, y)
		end
		return y
	end
	
	function frame.IsCheckBoxSelected(definition)
		local checked = frame.Settings[definition.Name]
		return checked == nil and definition.Default or checked
	end
	
	function frame.default()
		wipe(frame.Settings)
		frame.apply()
	end
	
	function frame.refresh()
		for definition, checkBox in u.Pairs(frame.CheckBoxes) do
			checkBox:SetChecked(frame.IsCheckBoxSelected(definition))
		end
		u.Call(frame.CustomRefresh)
	end
	
	function frame.okay()
		u.ShallowCopy(frame.Settings, frame.SavedSettings)
		frame.apply()
	end
	
	function frame.cancel()
		u.ShallowCopy(frame.SavedSettings, frame.Settings)
		frame.apply()
	end
	
	InterfaceOptions_AddCategory(frame)
	return frame
end
